@@ -36,13 +36,13 @@ def sanitize_name(name):
 def extra_pll_bels(cell, row, col, num, cellname):
     # rPLL can occupy several cells, add them depending on the chip
     offx = 1;
-    if device == 'GW1N-9C':
+    if device in {'GW1N-9C', 'GW1N-9'}:
         if int(col) > 28:
             offx = -1
         for off in [1, 2, 3]:
             yield ('RPLLB', int(row), int(col) + offx * off, num,
                 cell['parameters'], cell['attributes'], sanitize_name(cellname) + f'B{off}')
-    elif device in {'GW1N-1', 'GW1NZ-1'}:
+    elif device in {'GW1N-1', 'GW1NZ-1', 'GW1N-4'}:
         for off in [1]:
             yield ('RPLLB', int(row), int(col) + offx * off, num,
                 cell['parameters'], cell['attributes'], sanitize_name(cellname) + f'B{off}')
@@ -528,7 +528,7 @@ def place(db, tilemap, bels, cst, args):
             bits = set()
             if 'PLL' in db.shortval[tiledata.ttyp].keys():
                 bits = get_shortval_fuses(db, tiledata.ttyp, pll_attrs, 'PLL')
-            #print(typ, bits)
+            #print(typ, tiledata.ttyp, bits)
             for r, c in bits:
                 tile[r][c] = 1
         elif typ == 'PLLVR':
